@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -135,8 +136,8 @@ static gint query_editor_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 static void
 create_tags_for_sql (QueryEditor *editor, const gchar *language)
 {
-	GtkTextBuffer *buffer;
 #ifdef HAVE_GTKSOURCEVIEW
+	GtkTextBuffer *buffer;
 	GtkSourceLanguageManager *mgr;
 	GtkSourceLanguage *lang;
 	gchar ** current_search_path;
@@ -150,8 +151,8 @@ create_tags_for_sql (QueryEditor *editor, const gchar *language)
 	g_return_if_fail (language != NULL);
 	g_return_if_fail (!strcmp (language, QUERY_EDITOR_LANGUAGE_SQL));
 
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (editor->priv->text));
 #ifdef HAVE_GTKSOURCEVIEW
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (editor->priv->text));
 	mgr = gtk_source_language_manager_new ();
 
 	/* alter search path */
@@ -429,12 +430,13 @@ display_completions (QueryEditor *editor)
 				gtk_tree_selection_select_iter (gtk_tree_view_get_selection (editor->priv->completion_treeview),
 								&iter);
 
+			GtkRequisition nat_req;
 			g_object_set ((GObject*) editor->priv->completion_renderer, "text", compl[i], NULL);
-			gtk_cell_renderer_get_size (editor->priv->completion_renderer,
-						    (GtkWidget*) editor->priv->completion_treeview,
-						    NULL, NULL, NULL, &w, &h);
-			width = MAX (width, w);
-			height += h + 2;
+			gtk_cell_renderer_get_preferred_size (editor->priv->completion_renderer,
+							      (GtkWidget*) editor->priv->completion_treeview,
+							      NULL, &nat_req);
+			width = MAX (width, nat_req.width);
+			height += nat_req.height + 2;
 		}
 		g_strfreev (compl);
 		
@@ -657,6 +659,49 @@ text_view_draw (GtkTextView *tv, cairo_t *cr, QueryEditor *editor)
 	return FALSE;
 }
 
+static void
+copy_all_in_signle_line_cb (GtkMenuItem *mitem, QueryEditor *editor)
+{
+	gchar *all, *ptr;
+	GString *string;
+	GtkClipboard *clipboard;
+
+	all = query_editor_get_all_text (editor);
+	if (!all)
+		return;
+
+	string = g_string_new ("");
+	for (ptr = all; *ptr; ptr++) {
+		if (*ptr == '\n') {
+			if ((ptr > all) && (ptr[-1] != ' '))
+				g_string_append_c (string, ' ');
+		}
+		else {
+			if ((*ptr == '-') && (ptr[1] == '-')) {
+				/* comment => skip till end of line */
+				for (; *ptr && *ptr != '\n'; ptr++);
+			}
+			else
+				g_string_append_c (string, *ptr);
+		}
+	}
+	g_free (all);
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clipboard, string->str, -1);
+	g_string_free (string, TRUE);
+}
+
+static void
+text_view_populate_popup_cb (GtkTextView *entry, GtkMenu *menu, QueryEditor *editor)
+{
+	GtkWidget *mitem;
+	mitem = gtk_menu_item_new_with_label (_("Copy all in a single line"));
+	g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (copy_all_in_signle_line_cb),
+			  editor);
+        gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), mitem);
+        gtk_widget_show (mitem);
+}
+
 static void
 query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 {
@@ -726,6 +771,8 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 			  G_CALLBACK (text_buffer_changed_cb), editor);
 	g_signal_connect (editor->priv->text, "draw",
 			  G_CALLBACK (text_view_draw), editor);
+	g_signal_connect (editor->priv->text, "populate-popup",
+			  G_CALLBACK (text_view_populate_popup_cb), editor);
 
 	/* create some tags */
 	GtkTextBuffer *buffer;
@@ -1469,7 +1516,7 @@ query_editor_get_current_history_batch (QueryEditor *editor)
  * query_editor_history_is_empty
  * @editor: a #QueryEditor widget.
  *
- * Returns: %TRUE if @editor doe snot have any history item
+ * Returns: %TRUE if @editor does not have any history item
  */
 gboolean
 query_editor_history_is_empty (QueryEditor *editor)
